@@ -41,25 +41,39 @@
 #' appropriate \code{InitErgmTerm.X} function.}
 #' \item{etamap}{the theta -> eta mapping as a list returned from
 #' <ergm.etamap>}
+#' \item{term.options}{the term options used to initialize the terms}
 #' \item{uid}{a string generated with the model, \UIDalgo; different models are, generally, guaranteed to have different strings, but identical models are not guaranteed to have the same string}
 #' @seealso [summary.ergm_model()], [ergm_preprocess_response()]
 #' @keywords internal
 #' @export
-ergm_model <- function(formula, nw=NULL, silent=FALSE, ..., term.options=list(), extra.aux=list(), env=globalenv(), offset.decorate=TRUE, terms.only=FALSE){
-  if (!is(formula, "formula") && !is(formula, "term_list"))
-    stop("Invalid model specification of class ",sQuote(class(formula)),".", call.=FALSE)
-  
-  #' @importFrom statnet.common eval_lhs.formula
-  if(is.null(nw)) nw <- eval_lhs.formula(formula)
+ergm_model <- function(object, nw=NULL, ..., formula = NULL){
+  ## TODO: Remove workaround around ergm 4.9.
+  if(!is.null(formula)){
+    .Deprecate_once(msg = "ergm_model()'s first argument is now 'object'.")
+    object <- formula
+  }
+  UseMethod("ergm_model", object)
+}
 
+#' @describeIn ergm_model a method for [`formula`]: extracts the [`network`] and the [`term_list`] and passes it on to the next method.
+#' @export
+ergm_model.formula <- function(object, nw=NULL, ...){
+  #' @importFrom statnet.common eval_lhs.formula
+  if(is.null(nw)) nw <- eval_lhs.formula(object)
+
+  #' @importFrom statnet.common list_rhs.formula
+  v <- list_rhs.formula(object)
+
+  ergm_model(v, nw, ...)
+}
+
+#' @describeIn ergm_model a method for [`term_list`]: `nw=` is mandatory;  initializes the terms in the list and passes it on to the next method.
+#' @export
+ergm_model.term_list <- function(object, nw=NULL, silent=FALSE, ..., term.options=list(), env=globalenv(), extra.aux=list(), offset.decorate=TRUE, terms.only=FALSE){
+  v <- object
   nw <- ensure_network(nw)
   nw <- as.network(nw, populate=FALSE) # In case it's an ergm_state.
-  
-  #' @importFrom statnet.common list_rhs.formula
-  v <- if(is(formula, "formula")) list_rhs.formula(formula)
-       else if(is(formula, "term_list")) formula
-       else stop("Invalid format for formula= argument: must be either an R formula or a term_list object.")
-  
+
   model <- structure(list(terms = list()),
                      class = "ergm_model")
 
@@ -102,8 +116,22 @@ ergm_model <- function(formula, nw=NULL, silent=FALSE, ..., term.options=list(),
     for(outlist in subterms) model <- updatemodel.ErgmTerm(model, outlist, offset=offset, offset.decorate=offset.decorate, silent=silent)
   }
 
+  model$term.options <- as.list(getOption("ergm.term")) %>% modifyList(as.list(term.options)) %>% modifyList(list(...))
+  ergm_model(model, nw, extra.aux = extra.aux, offset.decorate = offset.decorate, terms.only = terms.only)
+}
+
+#' @describeIn ergm_model a method for [`term_list`]: `nw=` is mandatory, and `term.options=` must be a part of the [`ergm_model`] object, with `...` ignored; (re)generates the auxiliaries and, the eta map, and the unique ID; and decorates offsets.
+#' @export
+ergm_model.ergm_model <- function(object, nw, ..., env=globalenv(), extra.aux=list(), offset.decorate=TRUE, terms.only=FALSE){
+  model <- object
+
+  ## If initialized auxiliaries are present, get rid of them.
+  np <- nparam(model, byterm = TRUE, canonical = TRUE)
+  if(any(np != 0))
+    model$terms <- model$terms[np != 0]
+
   if(!terms.only){
-    model <- ergm.auxstorage(model, nw, term.options=term.options, ..., extra.aux=extra.aux)
+    model <- ergm.auxstorage(model, nw, term.options=model$term.options, extra.aux=extra.aux)
     model$etamap <- ergm.etamap(model)
     model$uid <- .GUID()
   }
@@ -129,7 +157,6 @@ ergm_model <- function(formula, nw=NULL, silent=FALSE, ..., term.options=list(),
   # Note that soname is not the same, since it's not guaranteed to be a loadable package.
   ergm.MCMC.packagenames(unlist(sapply(model$terms, "[[", "pkgname")))
 
-  class(model) <- "ergm_model"
   model
 }
 
@@ -168,6 +195,8 @@ call.ErgmTerm <- function(term, env, nw, ..., term.options=list()){
   # so that it will have access to any parameters of the ergm terms
   out <- eval(termCall, env)
   if(is.null(out)) return(NULL)
+  # If dyadic dependence is not specified, assume dependent.
+  NVL(out$dependence) <- TRUE
   # If SO package name not specified explicitly, autodetect.
   if(is.null(out$pkgname)) out$pkgname <- environmentName(environment(eval(termFun)))
   # Store the term call in the term list (with the term able to override)
@@ -292,8 +321,7 @@ c.ergm_model <- function(...){
     # New auxiliary list is o-nonsinks, m-nonsinks, and sinks.
     o$terms <- c(o$terms[!oaux], m$terms[!maux], o$terms[oaux][onaux!=0L], m$terms[maux][mnaux!=0L], sinks)
 
-    for(name in c("coef.names",
-                  "minval",
+    for(name in c("minval",
                   "maxval",
                   "offset",
                   "term.skipped",
@@ -309,6 +337,7 @@ c.ergm_model <- function(...){
   o
 }
 
+
 #' @rdname ergm_model
 #' @param x object to be converted to an `ergm_model`.
 #' @export
@@ -322,7 +351,7 @@ as.ergm_model.ergm_model <- function(x, ...) x
 #' @rdname ergm_model
 #' @export
 as.ergm_model.formula <- function(x, ...)
-  ergm_model(formula=x, ...)
+  ergm_model(object = x, ...)
 
 #' @noRd
 #' @export
