@@ -22,8 +22,8 @@
 #' statistics of the network, and, optionally, the final network.
 #' 
 #' 
-#' @param formula An [ergm()]-style formula, with a
-#'   [`network`] on its LHS.
+#' @param object An [ergm()]-style formula, with a [`network`] on its
+#'   LHS, an [ergm_model()] or the object appropriate to the method.
 #' @param changes Either a matrix with three columns: tail, head, and
 #'   new value, describing the changes to be made; or a list of such
 #'   matrices to apply these changes in a sequence. For binary network
@@ -42,10 +42,15 @@
 #' @param changes.only Whether to return network statistics or only
 #'   their changes relative to the initial network.
 #'
-#' @templateVar mycontrol control.ergm.godfather
-#' @template control
+#' @param control Deprecated; arguments such as `term.options` can be
+#'   passed directly.
+#' @param formula Deprecated; replaced with `object` for consistency.
 #' @template verbose
 #'
+#' @param ... additional arguments to [ergm_model()].
+#'
+#' @template basis
+#'
 #' @return If \code{end.network==FALSE} (the default), an
 #'   [`mcmc`] object with the requested network statistics
 #'   associed with the network series produced by applying the
@@ -70,42 +75,107 @@
 #'                             cbind(1:2,2:3)),
 #'                stats.start=TRUE)
 #' @export ergm.godfather
-ergm.godfather <- function(formula, changes=NULL, response=NULL,
+ergm.godfather <- function(object, changes=NULL,
+                           ...,
                            end.network=FALSE,
                            stats.start=FALSE,
                            changes.only=FALSE,
                            verbose=FALSE,
-                           control=control.ergm.godfather()){
-  on.exit(ergm_Cstate_clear())
+                           basis = NULL,
+                           formula = NULL){
+  ## TODO: Remove around ergm 4.9:
+  if(!is.null(formula)){
+    .Deprecate_once(msg = paste0("Argument ", sQuote("formula="), " to ", sQuote("ergm.godfather()"), "has been replaced with ", sQuote("formula="), "."))
+    object <- formula
+  }
 
-  check.control.class("ergm.godfather", "ergm.godfather")
+  UseMethod("ergm.godfather", object)
+}
+
+#' @rdname ergm.godfather
+#' @export
+ergm.godfather.formula <- function(object, changes=NULL, response=NULL,
+                                   ...,
+                                   end.network=FALSE,
+                                   stats.start=FALSE,
+                                   changes.only=FALSE,
+                                   verbose=FALSE,
+                                   control=NULL,
+                                   basis = ergm.getnetwork(object)){
+  ergm_preprocess_response(basis,response)
+  ## TODO: Remove this workaround around the 4.9 release.
+  m <- if("term.options" %in% ...names()) ergm_model(object, basis, ...)
+       else ergm_model(object, basis, term.options = control$term.options, ...)
+
+  ergm.godfather(m, changes = changes, ...,
+                 end.network = end.network,
+                 stats.start = stats.start,
+                 changes.only = changes.only,
+                 verbose = verbose,
+                 control = control,
+                 basis = basis)
+}
+
+#' @rdname ergm.godfather
+#' @note [ergm.godfather.ergm_model()] is a lower-level interface, providing
+#'   an [ergm.godfather()] method for the [`ergm_model`] class. The `basis`
+#'   argument is required.
+#' @export
+ergm.godfather.ergm_model <- function(object, changes=NULL,
+                                   ...,
+                                   end.network=FALSE,
+                                   stats.start=FALSE,
+                                   changes.only=FALSE,
+                                   verbose=FALSE,
+                                   control=NULL,
+                                   basis = NULL){
+  if(is.null(basis)) stop("This method requires the ", sQuote("basis="), " argument.")
+
+  state <- ergm_state(basis, model=object)
+  state <- update(state, stats = if(changes.only) numeric(nparam(state,canonical=TRUE)) else summary(state))
+
+  s <- ergm.godfather(state, changes = changes, ...,
+                      end.network = end.network, stats.start = stats.start,
+                      changes.only = changes.only, verbose = verbose,
+                      control = control)
+
+  if(end.network) structure(as.network(s), stats = attr(s, "stats"))
+  else s
+}
+
+#' @rdname ergm.godfather
+#' @note [ergm.godfather.ergm_model()] is a lower-level interface, providing
+#'   an [ergm.godfather()] method for the [`ergm_model`] class. The `basis`
+#'   argument is required.
+#' @export
+ergm.godfather.ergm_state <- function(object, changes=NULL,
+                                   ...,
+                                   end.network=FALSE,
+                                   stats.start=FALSE,
+                                   verbose=FALSE,
+                                   control=NULL){
+  if(!is.null(control)) check.control.class("ergm.godfather", "ergm.godfather")
 
   if(!is.list(changes)) changes <- list(changes)
 
-  nw <- ergm.getnetwork(formula)
-  ergm_preprocess_response(nw,response)
-
   ncols <- sapply(changes, ncol)
-  if(!all_identical(ncols) || ncols[1]<2 || ncols[1]>3 || (is.valued(nw)&&ncols[1]==2)) abort("Invalid format for list of changes. See help('ergm.godfather').")
-
-  m <- ergm_model(formula, nw, term.options=control$term.options)
-  state <- ergm_state(nw, model=m)
-  state <- update(state, stats = if(changes.only) numeric(nparam(state,canonical=TRUE)) else summary(state))
+  if(!all_identical(ncols) || ncols[1]<2 || ncols[1]>3 || (is.valued(object)&&ncols[1]==2)) abort("Invalid format for list of changes. See help('ergm.godfather').")
 
   changem <- changes %>% map(~rbind(0L,.)) %>% do.call(rbind, .) # 0s are sentinels indicating next iteration.
   if(!stats.start) changem <- changem[-1,,drop=FALSE] # I.e., overwrite the initial statistic rather than advancing past it first thing.
-  if(!is.directed(nw)) {
+  if(!is.directed(object$nw0)) {
     tails <- changem[,1]
     heads <- changem[,2]
     changem[,1] <- pmin(tails, heads)
     changem[,2] <- pmax(tails, heads)
   }
   
-  if(verbose) message_print("Applying changes...\n")
+  if(verbose) message("Applying changes...")
+  on.exit(ergm_Cstate_clear())
   z <-
-    if(!is.valued(state))
+    if(!is.valued(object))
       .Call("Godfather_wrapper",
-            state,
+            object,
             # Godfather settings
             as.integer(changem[,1]),
             as.integer(changem[,2]),
@@ -115,7 +185,7 @@ ergm.godfather <- function(formula, changes=NULL, response=NULL,
             PACKAGE="ergm")
     else
       .Call("WtGodfather_wrapper",
-            state,
+            object,
             # Godfather settings
             as.integer(changem[,1]),
             as.integer(changem[,2]),
@@ -124,28 +194,16 @@ ergm.godfather <- function(formula, changes=NULL, response=NULL,
             as.integer(verbose),
             PACKAGE="ergm")
 
-  stats <- matrix(z$s, ncol=nparam(m,canonical=TRUE), byrow=TRUE)
-  colnames(stats) <- param_names(m, canonical=TRUE)
+  stats <- matrix(z$s, ncol=nparam(object, canonical=TRUE), byrow=TRUE)
+  colnames(stats) <- param_names(object, canonical=TRUE)
 
   #' @importFrom coda mcmc
   stats <- mcmc(stats)
   
   if(end.network){ 
-    if(verbose) cat("Creating new network...\n")
-    newnetwork <- as.network(update(z$state))
-    attr(newnetwork,"stats")<-stats
+    if(verbose) message("Creating new network...")
+    newnetwork <- update(z$state)
+    attr(newnetwork,"stats") <- stats
     newnetwork
   }else stats
 }
-
-#' Control parameters for [ergm.godfather()].
-#'
-#' Returns a list of its arguments.
-#'
-#' @template term_options
-#' 
-#' @export control.ergm.godfather
-control.ergm.godfather<-function(term.options=NULL){
-  control <- handle.controls("control.ergm.godfather")
-  set.control.class("control.ergm.godfather")
-}
