@@ -5,11 +5,9 @@
 #  open source, and has the attribution requirements (GPL Section 7) at
 #  https://statnet.org/attribution .
 #
-#  Copyright 2003-2023 Statnet Commons
+#  Copyright 2003-2024 Statnet Commons
 ################################################################################
 #' @importFrom Rdpack reprompt
-#' @importFrom MASS ginv
-#' @importFrom Matrix nearPD
 .onAttach <- function(libname, pkgname){
   #' @importFrom statnet.common statnetStartupMessage
   sm <- statnetStartupMessage("ergm", c("statnet","ergm.count","tergm"), TRUE)
@@ -56,9 +54,9 @@ eval(UPDATE_MY_SCTRL_EXPR)
 
 .RegisterProposals <- function(){
   ergm_proposal_table("c", "Bernoulli", "|.dyads|bd",  -2, "random", "randomtoggle")
-  ergm_proposal_table("c", "Bernoulli", "|.dyads|bd&sparse",  -1, "TNT", "TNT")
+  ergm_proposal_table("c", "Bernoulli", "|.dyads|bd&sparse",  0, "TNT", "TNT")
   ergm_proposal_table("c", "Bernoulli", "|bdmax|blocks|strat&sparse",  -3, "BDStratTNT", "BDStratTNT")
-  ergm_proposal_table("c", "Bernoulli", c("&bdmax|blocks|strat&sparse", "|bdmax&blocks|strat&sparse", "|bdmax|blocks&strat&sparse"),  0, "BDStratTNT", "BDStratTNT")
+  ergm_proposal_table("c", "Bernoulli", c("&bdmax|blocks|strat&sparse", "|bdmax&blocks|strat&sparse", "|bdmax|blocks&strat&sparse"),  5, "BDStratTNT", "BDStratTNT")
   ergm_proposal_table("c", "Bernoulli", "", -100, "TNT10", "TNT10")
   ergm_proposal_table("c", "Bernoulli", "&degrees",  0, "random", "CondDegree")
   ergm_proposal_table("c", "Bernoulli", "&degreesmix",  0, "random", "CondDegreeMix")
@@ -75,6 +73,7 @@ eval(UPDATE_MY_SCTRL_EXPR)
   ergm_proposal_table("c", "Bernoulli", "&hamming&sparse",  0, "random", "HammingTNT")
   ergm_proposal_table("c", "Bernoulli", "&dyadnoise&sparse",  1, "TNT", "dyadnoiseTNT")
   ergm_proposal_table("c", "Bernoulli", "&dyadnoise",  0, "random", "dyadnoise")
+  ergm_proposal_table("c", "Bernoulli", "|.dyads|bd&sparse&triadic",  0, "TNT", "SPDyad")
 
   ergm_proposal_table("c", "StdNormal", "",  0, "random", "StdNormal")
   ergm_proposal_table("c", "StdNormal", "|.dyads",  0, "random", "DistRLE")
