@@ -5,7 +5,7 @@
  *  open source, and has the attribution requirements (GPL Section 7) at
  *  https://statnet.org/attribution .
  *
- *  Copyright 2003-2023 Statnet Commons
+ *  Copyright 2003-2024 Statnet Commons
  */
 #include "ergm_dyad_hashmap.h"
 #include "ergm_changestat.h"
@@ -23,6 +23,18 @@ void PrintDyadMapUInt(StoreDyadMapUInt *h){
   }
 }
 
+/* Print the contents of a khash table  mapping dyads to unsigned
+   integers. Useful for debugging. */
+void PrintStrictDyadMapUInt(StoreStrictDyadMapUInt *h){
+  for(khiter_t i = kh_begin(h); i!=kh_end(h); ++i){
+    if(kh_exist(h, i)){
+      TailHead k = kh_key(h, i);
+      unsigned int v = kh_val(h, i);
+      Rprintf("(%d,%d)->%u\n",k.tail,k.head,v);
+    }
+  }
+}
+
 /* Print the contents of a khash set of dyads. Useful for debugging. */
 void PrintDyadSet(StoreDyadSet *h){
   for(khiter_t i = kh_begin(h); i!=kh_end(h); ++i){
@@ -34,6 +46,17 @@ void PrintDyadSet(StoreDyadSet *h){
   Rprintf("\n");
 }
 
+/* Print the contents of a khash set of dyads. Useful for debugging. */
+void PrintStrictDyadSet(StoreStrictDyadSet *h){
+  for(khiter_t i = kh_begin(h); i!=kh_end(h); ++i){
+    if(kh_exist(h, i)){
+      TailHead k = kh_key(h, i);
+      Rprintf("(%d,%d) ",k.tail,k.head);
+    }
+  }
+  Rprintf("\n");
+}
+
 /* Copy network to a khash set of dyads. */
 StoreDyadSet *NetworkToDyadSet(Network *nwp){
   StoreDyadSet *h = kh_init(DyadSet);
@@ -44,3 +67,12 @@ StoreDyadSet *NetworkToDyadSet(Network *nwp){
     });
   return h;
 }
+
+StoreStrictDyadSet *NetworkToStrictDyadSet(Network *nwp){
+  StoreStrictDyadSet *h = kh_init(StrictDyadSet);
+
+  EXEC_THROUGH_NET_EDGES(tail, head, e, {
+      kh_put(StrictDyadSet, h, TH(tail,head), NULL);
+    });
+  return h;
+}
