@@ -5,19 +5,22 @@
 #  open source, and has the attribution requirements (GPL Section 7) at
 #  https://statnet.org/attribution .
 #
-#  Copyright 2003-2023 Statnet Commons
+#  Copyright 2003-2024 Statnet Commons
 ################################################################################
 
+set.seed(0)
+
 net1 <- network.initialize(10,directed=FALSE)
 net1[,] <- 1
 absent <- as.edgelist(net1)[sample.int(network.edgecount(net1), 2), ]
 net1[absent] <- 0
 present <- as.edgelist(net1)[sample.int(network.edgecount(net1), 2), ]
+fixed <- rbind(present, absent)
 
 net1[as.edgelist(net1)[sample.int(network.edgecount(net1), round(network.edgecount(net1)/2)), ]] <- 0
 net1[present] <- 1
 
-test_that("fixedas", {
+test_that("fixedas(present, absent)", {
   t1 <- ergm(net1~edges, constraint = ~fixedas(present = present, absent = absent))
   s1 <- simulate(t1, 100)
 
@@ -28,24 +31,44 @@ test_that("fixedas", {
   expect_true(all(!sapply(s1,function(x)as.data.frame(t(absent)) %in% as.data.frame(t(as.edgelist(x))))))
 })
 
+
+test_that("fixedas(fixed.dyads)", {
+  t1 <- ergm(net1~edges, constraint = ~fixedas(fixed))
+  s1 <- simulate(t1, 100)
+
+  # check that fixed edges are identical between simulated networks and the original network
+  expect_true(all(sapply(s1,function(x) identical(x[fixed], net1[fixed]))))
+})
+
+
 test_that("only present", {
   t1 <- ergm(net1~edges, constraint = ~fixedas(present = present))
   s1 <- simulate(t1,100)
   expect_true(all(sapply(s1,function(x)as.data.frame(t(present)) %in% as.data.frame(t(as.edgelist(x))))))
+
+  # Also test for inconsistent constraint.
+  expect_error(ergm(net1~edges, constraint = ~fixedas(present = absent)),
+               "In constraint 'fixedas' in package 'ergm': Edges constrained to be present are absent in the LHS network.")
 })
 
 test_that("only absent", {
-t1 <- ergm(net1~edges, constraint = ~fixedas(absent = absent))
-s1 <- simulate(t1, 100)
-expect_true(all(!sapply(s1,function(x)as.data.frame(t(absent)) %in% as.data.frame(t(as.edgelist(x))))))
+  t1 <- ergm(net1~edges, constraint = ~fixedas(absent = absent))
+  s1 <- simulate(t1, 100)
+  expect_true(all(!sapply(s1,function(x)as.data.frame(t(absent)) %in% as.data.frame(t(as.edgelist(x))))))
+
+  # Also test for inconsistent constraint.
+  expect_error(ergm(net1~edges, constraint = ~fixedas(absent = present)),
+               "In constraint 'fixedas' in package 'ergm': Edges constrained to be absent are present in the LHS network.")
 })
 
 present <- as.network(present, matrix.type = "edgelist", directed = FALSE)
 absent <- as.network(absent, matrix.type = "edgelist", directed = FALSE)
 
 test_that("fixedas with network input", {
-  t1 <- ergm(net1~edges, constraint = ~fixedas(present = present, absent = absent))
-  s1 <- simulate(t1, 100)
+  expect_warning(t1 <- ergm(net1~edges, constraint = ~fixedas(present = present, absent = absent)),
+                 "^In constraint 'fixedas' in package 'ergm': Network size of argument\\(s\\) 'present' and 'absent' differs from that of the response network\\..*")
+  expect_warning(s1 <- simulate(t1, 100),
+                 "^In constraint 'fixedas' in package 'ergm': Network size of argument\\(s\\) 'present' and 'absent' differs from that of the response network\\..*")
 
   expect_true(all(sapply(s1,function(x)as.data.frame(t(as.edgelist(present))) %in% as.data.frame(t(as.edgelist(x))))))
   expect_true(all(!sapply(s1,function(x)as.data.frame(t(as.edgelist(absent))) %in% as.data.frame(t(as.edgelist(x))))))
